using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Driver
{
    public class Driver : Form
    {
        public Device device = null;

        private Mesh roadMesh = null;
        private Material[] roadMaterials = null;
        private Texture[] roadTextures = null;

        public const float RoadLocationLeft = 2.5f;
        public const float RoadLocationRight = -2.5f;
        public const float RoadSize = 100.0f;
        public const float MaximumRoadSpeed = 250.0f;
        public const float MinimumRoadSpeed = 0.1f;
        public const float RoadSpeedIncrement = 0.5f;

        public float RoadDepth0 = 0.0f;
        public float RoadDepth1 = -100.0f;
        public float RoadSpeed = 0.0f;


        private float elapsedTime = 0.0f;

        private Car car = null;

        private Obstacle obstacles;


        public Driver()
        {
            this.ClientSize = new System.Drawing.Size(800, 600);

            this.Text = "Driver | Phantom's game";

            this.SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);

        }

        private void InitializeGraphics()
        {
            PresentParameters prPar = new PresentParameters();

            prPar.Windowed = true;
            prPar.SwapEffect = SwapEffect.Discard;
            prPar.AutoDepthStencilFormat = DepthFormat.D16;
            prPar.EnableAutoDepthStencil = true;

            int adapterOrdinal = Manager.Adapters.Default.Adapter;
            CreateFlags flags = CreateFlags.SoftwareVertexProcessing;

            Caps caps = Manager.GetDeviceCaps(adapterOrdinal, DeviceType.Hardware);

            if (caps.DeviceCaps.SupportsHardwareTransformAndLight)
            {
                flags = CreateFlags.HardwareVertexProcessing;
            }

           // if (caps.DeviceCaps.SupportsPureDevice)
           // {
           //     flags = CreateFlags.PureDevice;
           // }

            device = new Device(adapterOrdinal, DeviceType.Hardware, this, flags, prPar);

            device.DeviceReset += new EventHandler(this.OnDeviceReset);
            this.OnDeviceReset(device, null);
        }

        private void OnDeviceReset(object sender, EventArgs e)
        {
            device.Transform.Projection = Matrix.PerspectiveFovLH((float)Math.PI / 4.0f, this.Width / this.Height, 1.0f, 1000.0f);
            device.Transform.View = Matrix.LookAtLH(new Vector3(0.0f, 9.5f, 17.0f), new Vector3(), new Vector3(0, 1, 0));

            if (device.DeviceCaps.VertexProcessingCaps.SupportsDirectionalLights && (device.DeviceCaps.MaxActiveLights > 1))
            {
                //Our graphic card supports directional lightning

                //First directional light
                device.Lights[0].Type = LightType.Directional;
                device.Lights[0].Diffuse = Color.White;
                device.Lights[0].Direction = new Vector3(1, -1, -1);
                device.Lights[0].Enabled = true;


                //Second directional light
                device.Lights[1].Type = LightType.Directional;
                device.Lights[1].Diffuse = Color.White;
                device.Lights[1].Direction = new Vector3(-1, 1, -1);
                device.Lights[1].Enabled = true;
            } else { 
                // We can use just ambient lights

                device.RenderState.Ambient = Color.White;

            }

            if (device.DeviceCaps.TextureFilterCaps.SupportsMinifyAnisotropic)
            {
                device.SamplerState[0].MinFilter = TextureFilter.Anisotropic;
            }
            else if (device.DeviceCaps.TextureFilterCaps.SupportsMinifyLinear)
            {
                device.SamplerState[0].MinFilter = TextureFilter.Linear;
            }

            if (device.DeviceCaps.TextureFilterCaps.SupportsMagnifyAnisotropic)
            {
                device.SamplerState[0].MagFilter = TextureFilter.Anisotropic;
            }
            else if (device.DeviceCaps.TextureFilterCaps.SupportsMagnifyLinear)
            {
                device.SamplerState[0].MagFilter = TextureFilter.Linear;
            }

            //Load road mesh
            roadMesh = LoadMesh(device, "road.x", ref roadMaterials, ref roadTextures);

            car = new Car(device);

        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0);
            OnFrameUpdate();

            device.BeginScene();

            DrawRoad(0.0f, 0.0f, RoadDepth0);
            DrawRoad(0.0f, 0.0f, RoadDepth1);

            car.Draw(device);

            device.EndScene();

            device.Present();

            this.Invalidate();
        }

        private void OnFrameUpdate()
        {
            //elapsedTime = Utility.Timer(DirectXTimer.GetElapsedTime);
            elapsedTime = Environment.TickCount / 900000000.0f;

            car.Update(elapsedTime);


            RoadDepth0 += (RoadSpeed * elapsedTime);
            RoadDepth1 += (RoadSpeed * elapsedTime);


            if (RoadDepth0 > 75.0f)
            {
                RoadDepth0 = RoadDepth1 - 99.0f;
            }

            if (RoadDepth1 > 75.0f)
            {
                RoadDepth1 = RoadDepth0 - 99.0f;
            }
        }

        public static Mesh LoadMesh(Device device, string file, ref Material[] meshMaterials, ref Texture[] meshTextures)
        {
            ExtendedMaterial[] mtrl;

            //load mesh
            Mesh tempMesh = Mesh.FromFile(file, MeshFlags.Managed, device, out mtrl);

            if ((mtrl != null) && (mtrl.Length > 0))
            {
                meshMaterials = new Material[mtrl.Length];
                meshTextures  = new Texture[mtrl.Length];

                for (int i = 0; i < mtrl.Length; i++)
                {
                    meshMaterials[i] = mtrl[i].Material3D;

                    if ((mtrl[i].TextureFilename != null) && (mtrl[i].TextureFilename != string.Empty))
                    {
                        meshTextures[i] = TextureLoader.FromFile(device, mtrl[i].TextureFilename);
                    }
                }
            }

            return tempMesh;
        }

        private void DrawRoad(float x, float y, float z)
        {
            device.Transform.World = Matrix.Translation(x, y, z);
            for (int i = 0; i < roadMaterials.Length; i++)
            {
                device.Material = roadMaterials[i];
                device.SetTexture(0, roadTextures[i]);
                roadMesh.DrawSubset(i);
            }
        
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape) 
            {
                this.Close();
                return;
            }

            if (e.KeyCode == Keys.Up)
            {
                if (RoadSpeed <= MaximumRoadSpeed)
                {
                    RoadSpeed += 1.0f;
                }
            }


            if (e.KeyCode == Keys.Down)
            {
                if (RoadSpeed >= MinimumRoadSpeed) 
                {
                    RoadSpeed -= 0.5f;
                }
            }

            if (RoadSpeed >= 5.0f)
            {
                if (e.KeyCode == Keys.Left)
                {
                    car.IsMovingLeft = true;
                    car.IsMovingRight = false;
                }

                if (e.KeyCode == Keys.Right)
                {
                    car.IsMovingLeft = false;
                    car.IsMovingRight = true;
                }
            }

       }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left)
            {
                car.IsMovingLeft = false;
            }

            if (e.KeyCode == Keys.Right)
            {
                car.IsMovingRight = false;
            }
        }

        static void Main()
        {
            using (Driver prgrm = new Driver())
            {
                prgrm.Show();
                prgrm.InitializeGraphics();
                Application.Run(prgrm);
            }

        }
    }
}